#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 17-2-21
# @Author  : JCai
# @desc    :

from django.db import connection
from datetime import datetime
from django.db.models import Q
from django.utils import timezone

from kaifa.models_starpro import SysproductSkuProduct
from order_statistic.models import SkuDaysStatistic, SpuDaysStatistic
from sale_aliexpress.utils import get_dict_from_object
from sale_amazon.models.listing import AmazonListingProduct
from sysproduct.models import SpuComment
from starpro.utils import json_encode
from sysproduct.models import Category, ProjectTeamUser, SpuProduct, SkuProduct, ProjectTeam
from sysproduct.models.product import SaleSpuProduct
from warehouse.models import SpuInventory, SkuInventory,SkuWarehouseInventory,Warehouse
import time

def get_product_category(path=''):
    """获取品类树"""
    if path:
        categorys=Category.objects.filter(id=path)
        cate=categorys[0]
        if cate.category_id_path !='0':
            parent_path=cate.category_id_path+'>'+str(cate.id)
            categorys=Category.objects.filter(category_id_path=parent_path)
        else:
            categorys=Category.objects.filter(category_id_path=str(cate.id))

    else:
        categorys=Category.objects.filter(category_id_path='0')
    categoryInfos=[]
    for cate in categorys:
        if path:
            tmp = Category.objects.filter(category_id_path=cate.category_id_path+'>'+str(cate.id))
        else:
            tmp = Category.objects.filter(category_id_path=str(cate.id))
        categoryInfo={
                'id':cate.id,
                'name':cate.category_name,
                'isParent':True if len(tmp)>0 else False,
                }
        categoryInfos.append(categoryInfo)
    return categoryInfos


def brow_products(edit_info, request, position=''):

    user_id = request.user.id

    #amazon kucunlist add by 20170710 start
    if position == 'manager':
        project_code = edit_info.get('projectCode', '')
        get_stock = str(edit_info.get('stock', ''))
        if get_stock == '0':
            query_stock = Q(spuinventory__isnull=False) & ~Q(spuinventory__mg_on_hand=0)
        elif get_stock == '1':
            query_stock = Q(spuinventory__isnull=False) & ~Q(spuinventory__sz_on_hand=0)
        elif get_stock == '2':
            query_stock = Q(spuinventory__isnull=False) & ~Q(spuinventory__gz_on_hand=0)
        elif get_stock == '3':
            query_stock = Q(spuinventory__isnull=False) & ~Q(spuinventory__ch_on_hand=0)
        else:
            query_stock = Q()
        query_isnull = Q(project_team_id__in=[project_code]) & Q(push_state=1)
        query_nonull = Q(push_state=1) & Q(sale_ok = 1) & Q(purchase_ok = 1)
        if project_code:
            products = SpuProduct.objects\
                .filter(query_isnull & query_stock) \
                .order_by('-create_date')
        else:
            products = SpuProduct.objects\
                .filter(query_nonull & query_stock) \
                .order_by('-create_date')
    else:
        teams = ProjectTeamUser.objects.filter(user_id=user_id)
        project_code = edit_info.get('projectCode', '')
        if project_code:
            team_info = ProjectTeam.objects.filter(id=project_code, state=0).first()
            if not team_info:
                products = SpuProduct.objects.filter()
                cate_data = SaleSpuProduct.get_category_products_count(products)
                return {'rows': [], 'total': 0, 'cate_data': cate_data}

            products = SpuProduct.objects\
                .filter(project_team_id__in=[project_code], push_state=1,
                        ) \
                .order_by('-create_date')
        else:
            team_nums = ProjectTeam.objects.filter(id__in=[x.team_id for x in teams], state=0)
            products = SpuProduct.objects\
                .filter(project_team_id__in=[x.id for x in team_nums], push_state=1,
                        sale_ok = 1, purchase_ok = 1) \
                .order_by('-create_date')

    product_search = edit_info.get('product', '')
    if product_search == '1':
        day_now = timezone.now()
        start_date = day_now - timezone.timedelta(hours=day_now.hour, minutes=day_now.minute,
                                                 seconds=day_now.second, microseconds=day_now.microsecond)
        day_end = start_date + timezone.timedelta(hours=23, minutes=59, seconds=59, microseconds=999999)
        day_start = start_date + timezone.timedelta(days=-29)
        products = products.filter(create_date__range=(day_start, day_end))
    elif product_search == '4':
        products = products.filter(product_level=6)
    elif product_search == '5':
        products_comment_spu = SpuComment.objects.values_list('spu',flat=True).distinct()
        products = products.filter(spu__in=products_comment_spu)

    developer_type = edit_info.get('type', '')
    if developer_type and developer_type != '0':
        products = products.filter(developer_type=developer_type)

    product_key = edit_info.get('title', '').strip()# 全局
    search_type = edit_info.get('searchStatus', 'searchLike')
    if search_type == 'pinlei':  # 找品类对应
        sys_sku = AmazonListingProduct.objects.filter().values('sysproduct_sku')
        sys_spu = SysproductSkuProduct.objects.filter(id__in=[x['sysproduct_sku'] for x in sys_sku]).values('spu_code')
        products = products.filter(spu__in=[x['spu_code'] for x in sys_spu])
        if product_key:
            products = products.filter(spu__icontains=product_key)
    if product_key:
        if search_type == 'searchLike':  # 模糊查询条件
            if product_key > u'\u4e00' and product_key < u'\u9fa5':
                products = products.filter(chinese_title__icontains=product_key)
            else:
                products = products.filter(Q(spu__icontains=product_key)
                                           | Q(chinese_title__icontains=product_key)
                                           | Q(title__icontains=product_key))

        elif search_type == 'nameCNLike':  # 按中文名称
            products = products.filter(chinese_title__icontains=product_key)
        elif search_type == 'nameENLike':  # 按英文名称
            products = products.filter(title__icontains=product_key)
        elif search_type == 'virtualSpu':  # 按虚拟SPU
            products = products.filter(spu__icontains=product_key)
        elif search_type == 'virtualSku':  # 按虚拟SKU
            # product_key = edit_info.get('title', '').strip().replace(' ','').split(',')  #
            product_key = edit_info.get('title', '').split(' ')
            product_key_list = []
            for tem_product_key in product_key:
                tem_product_key = tem_product_key.strip()
                product_key_list.append(tem_product_key)
            sku_obj = SkuProduct.objects.filter(sku__in=product_key_list).exclude(product_lifecycle=4)
            #sku_obj = SkuProduct.objects.filter(sku=product_key).exclude(product_lifecycle=4).first()
            if sku_obj:
                spu_code_list=[]
                for tem_sku_obj in sku_obj:
                    spu_code_list.append(tem_sku_obj.spu_code)
                products = products.filter(spu__in = spu_code_list)
            else:
                products = SpuProduct.objects.none()

    daterange = edit_info.get('daterange', None)
    if daterange:
        dateranges = str(daterange).split(' to ')
        if len(dateranges) >= 2:
            start_date = datetime.strptime(dateranges[0], '%Y-%m-%d')
            end_date = datetime.strptime(dateranges[1], '%Y-%m-%d')
            products = products.filter(create_date__range=(start_date, end_date))


    category_select = edit_info.get('cate_id', '')
    limit = int(edit_info.get('limit', '50'))
    offset = int(edit_info.get('offset', '0'))
    if offset == 0:
        cate_data = SaleSpuProduct.get_category_products_count(products)
    else:
        cate_data = []
    if category_select:
        cate = Category.objects.filter(id=category_select).first()
        products = products.filter(
            category_id_path__startswith=cate.category_id_path + '>' + category_select + '>' if cate.category_id_path != '0' else category_select + '>')

    day_now = timezone.now()
    start_date = day_now - timezone.timedelta(hours=day_now.hour, minutes=day_now.minute,
                                              seconds=day_now.second, microseconds=day_now.microsecond)
    day_end = start_date + timezone.timedelta(hours=23, minutes=59, seconds=59, microseconds=999999)
    day_start = start_date + timezone.timedelta(days=-29)
    end_timestamp = time.mktime(time.strptime(str(day_end)[:19], '%Y-%m-%d %H:%M:%S'))
    start_timestamp = time.mktime(time.strptime(str(day_start)[:19], '%Y-%m-%d %H:%M:%S'))
    total = products.count()
    items = []
    for _item in products[offset:offset + limit]:
        _info = {}
        _info['projectCode'] = _item.project_code
        _info['spu'] = _item.spu
        _info['title'] = _item.title
        _info['chinese_title'] = _item.chinese_title
        _info['gross_weight'] = _item.gross_weight
        _info['product_level'] = _item.product_level
        _info['product_lifecycle'] = _item.product_lifecycle
        _info['price'] = _item.price
        _info['create_date'] = eval(json_encode(_item.create_date))

        product_flag = []
        create_date_timestamp = time.mktime(time.strptime(str(_info['create_date'])[:19], '%Y-%m-%d %H:%M:%S'))
        if create_date_timestamp > start_timestamp and create_date_timestamp < end_timestamp:
            product_flag.append(1)
        else:
            product_flag.append(0)
        if _item.product_level == 6:
            product_flag.append(1)
        else:
            product_flag.append(0)
        _info['product_flag'] = product_flag

        try:
            _info['product_manager'] = _item.product_manager.get_cn_full_name() if _item.product_manager else ''
        except:
            _info['product_manager'] = '--'

        try:
            spu_inventory = SpuInventory.objects.filter(spu_id=_item.id).first()
            if spu_inventory:
                _info['sz'] = spu_inventory.sz_on_hand
                _info['us'] = spu_inventory.mg_on_hand
                #11111111111111111111111111111111111111111111111111111111
                _info['gz'] = spu_inventory.gz_on_hand
                _info['ch'] = spu_inventory.ch_on_hand
        except:
            pass

        _info['main_image'] = _item.main_image

        _info['product_list'] = []
        skus = _item.skuproduct_set.filter().exclude(product_lifecycle=4)
        for sku in skus:
            product_sku = {'weight': sku.get_weight(),
                           'price': sku.get_price(),
                           'img': sku.main_image,
                           'sku': sku.sku,
                           'attr_info': '<br>'.join([each[0]+': '+each[1] for each in sku.get_attr_info().items()])}
            try:
                sku_inventory = SkuInventory.objects.filter(sku_id__sku=sku.sku).first()
            except:
                pass
            if sku_inventory:
                product_sku['sz'] = sku_inventory.sz_on_hand
                product_sku['us'] = sku_inventory.mg_on_hand
                statistic_month_sku = SkuDaysStatistic.objects.filter(sku=sku_inventory.sku_id.id).first()
                if statistic_month_sku:
                    product_sku['month'] = statistic_month_sku.month_qty
            _info['product_list'].append(product_sku)

        _info['count'] = len(_info['product_list'])

        month_spu = SpuDaysStatistic.objects.filter(spu=_item.id).first()
        if month_spu:
            _info['month'] = month_spu.month_qty

        items.append(_info)

    return {'rows': items, 'total': total, 'cate_data': cate_data}

def get_inventory_by_skuid(sku_id):
    data=SkuWarehouseInventory.objects.filter(sku_id=sku_id)
    data_list=[]
    for each in data:
        each_dict=get_dict_from_object(each)
        print each_dict
        house_obj=Warehouse.objects.filter(warehouse_id=str(each.warehouse_id_id)).first()
        each_dict['warehouse_name']=house_obj.warehouse_name
        data_list.append(each_dict)
    return data_list


def get_permition(user_id,sku,spu, position=''):
    if position == 'manager':
        if sku:
            sku_obj = SkuProduct.objects.filter(sku=sku).first()
            p = SpuProduct.objects.filter(id=sku_obj.spu.id)
        else:
            p = SpuProduct.objects.filter(spu=spu)
    else:
        if sku:
            sku_obj = SkuProduct.objects.filter(sku=sku).first()
            teams = ProjectTeamUser.objects.filter(user_id=user_id)
            team_nums = ProjectTeam.objects.filter(id__in=[x.team_id for x in teams], state=0)
            p = SpuProduct.objects.filter(id=sku_obj.spu.id, project_team_id__in=[x.id for x in team_nums])
        else:
            teams = ProjectTeamUser.objects.filter(user_id=user_id)
            team_nums = ProjectTeam.objects.filter(id__in=[x.team_id for x in teams], state=0)
            p = SpuProduct.objects.filter(spu=spu, project_team_id__in=[x.id for x in team_nums])

    if p:
        return True
    else:
        return False

